from django.apps import AppConfig


class AmazonTranslateConfig(AppConfig):
    """
    Amazon Translate Django Application

    Utilities for models and command line to make use of Amazon Translate
    to be able to automatically translate between English and required
    languages.
    """
    name = 'django_amazon_translate'



from modeltranslation.decorators import register
from modeltranslation.translator import TranslationOptions

from .models import TranslatedModel


@register(TranslatedModel)
class TranslationOptions(TranslationOptions):
    fields = ('title', 'description',)



import re

import boto3
from django.db import models
from modeltranslation.settings import AVAILABLE_LANGUAGES, DEFAULT_LANGUAGE
from modeltranslation.translator import translator
from modeltranslation.utils import build_localized_fieldname


class _amazonTranslate(object):
    def __init__(self, client=None):
        self.translate = client or boto3.client('translate')

    def translate_text(self, text, source_language, dest_language):
        return self.translate.translate_text(
                            Text=text,
                            SourceLanguageCode=source_language,
                            TargetLanguageCode=dest_language
        )


def get_translate_client():
    return _amazonTranslate()


def translate_po_file(po, language):
    """
    Update a given .po file with translated strings from Amazon Translate.
    """

    translate = get_translate_client()

    for s in po.untranslated_entries():
        # We replace the formatting specifiers with something
        # that Amazon Translate will just assume is a title and
        # not translate.
        subbed_message = re.sub(
            r"%\((\w+)\)s", r"FORMAT_\1_END", s.msgid
        )

        response = translate.translate_text(
            subbed_message,
            "en",
            language,
        )

        # Put back the correct gettext formatting
        s.msgstr = re.sub(
            r"FORMAT_(\w+)_END", r"%(\1)s",
            response['TranslatedText']
        )

    return po


class TranslationTracker(object):
    """
    Django Model addition/plugin to provide automated translations
    in conjunction with modeltranslation and Amazon Translate
    """

    def contribute_to_class(self, cls, name):
        self.name = name

        models.signals.class_prepared.connect(self.finalize_class, sender=cls)

    def finalize_class(self, sender, **kwargs):
        self.model_class = sender
        setattr(sender, self.name, self)

        self.patch_save(sender)

    @staticmethod
    def patch_save(model):
        original_save = model.save

        def save(instance, *args, **kwargs):
            translate = get_translate_client()

            cls = instance.__class__
            old = cls.objects.filter(pk=instance.pk).first()
            new = instance

            opts = translator.get_options_for_model(cls)

            # Loop through the fields and work out what actually changed
            changed_fields = []
            if old:
                for field in cls._meta.get_fields():
                    field_name = field.name

                    if getattr(old, field_name) != getattr(new, field_name):
                        changed_fields.append(field_name)

            # Go through each translatable field
            for field_name in opts.fields.keys():
                def_lang_fieldname = build_localized_fieldname(
                    field_name, DEFAULT_LANGUAGE
                )

                for lang in AVAILABLE_LANGUAGES:
                    if lang == DEFAULT_LANGUAGE:
                        continue

                    lang_fieldname = build_localized_fieldname(
                        field_name, lang
                    )

                    # If they updated the translation as well as
                    # the source.
                    if lang_fieldname in changed_fields:
                        continue

                    if (
                        def_lang_fieldname in changed_fields
                        or not getattr(new, def_lang_fieldname)
                        or not old
                    ):
                        r = translate.translate_text(
                            getattr(new, def_lang_fieldname),
                            DEFAULT_LANGUAGE,
                            lang
                        )
                        setattr(new, lang_fieldname, r['TranslatedText'])

            return original_save(instance, *args, **kwargs)

        model.save = save



import polib

from django_amazon_translate.management.commands.auto_translate_text \
    import _language_from_path
from django_amazon_translate.translator import translate_po_file

pofile = """
# SOME DESCRIPTIVE TITLE.
# Copyright (C) YEAR THE PACKAGE'S COPYRIGHT HOLDER
# This file is distributed under the same license as the PACKAGE package.
# FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.
#
#, fuzzy
msgid ""
msgstr ""
"Project-Id-Version: PACKAGE VERSION\n"
"Report-Msgid-Bugs-To: \n"
"POT-Creation-Date: 2019-05-23 12:46+0000\n"
"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language-Team: LANGUAGE <LL@li.org>\n"
"Language: \n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=UTF-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Plural-Forms: nplurals=2; plural=(n > 1);\n"

#: ham/egg/models.py:1
msgid "String to translate"
msgstr ""

#: ham/egg/models.py:2
msgid "String that is already translated"
msgstr "Chaîne déjà traduite"
"""


def test_translatepofile(mocker):
    x = mocker.patch(
        'django_amazon_translate.translator._amazonTranslate.translate_text',
        return_value={'TranslatedText': 'Nothing'}
    )

    po = polib.pofile(pofile)
    po = translate_po_file(po, "fr")

    x.assert_called_once()


def test_extractlanguage():
    assert(
        _language_from_path(
            'LOCALE_PATH',
            'LOCALE_PATH/en/LC_MESSAGES/django.po'
        ) == 'en'
    )



import os
from pathlib import Path

import polib
from django.conf import settings
from django.core.management import BaseCommand

from django_amazon_translate.translator import translate_po_file


def _language_from_path(locale_path, path):
    p = str(path).replace(str(locale_path) + os.path.sep, "")
    return p[:p.find(os.path.sep)]


class Command(BaseCommand):
    help = 'Use Amazon Translate to translate all the .po files in' \
           ' LOCALE_PATHS, any translated strings are not touched'

    def handle(self, *args, **options):
        # Loop through all locale paths defined in the project
        for f in settings.LOCALE_PATHS:
            # Find all the .po files we can
            root_dir = Path(f)
            file_list = [f for f in root_dir.glob('**/*.po')]

            for entry in file_list:
                language = _language_from_path(f, entry)

                po = polib.pofile(entry)
                po = translate_po_file(po, language)
                po.save(entry)



from unittest.mock import call

from django_amazon_translate.tests.testapp.models import TranslatedModel


def test_translatedmodel(mocker):
    x = mocker.patch(
        'django_amazon_translate.translator._amazonTranslate.translate_text',
        return_value={'TranslatedText': 'Nothing'}
    )

    f = TranslatedModel()
    f.title_en = "Ham"
    f.description_en = "Eggs"
    f.save()

    calls = [
                call('Ham', 'en', 'de'),
                call('Ham', 'en', 'fr'),
                call('Ham', 'en', 'it'),
                call('Ham', 'en', 'es'),
                call('Eggs', 'en', 'de'),
                call('Eggs', 'en', 'fr'),
                call('Eggs', 'en', 'it'),
                call('Eggs', 'en', 'es'),
            ]

    x.assert_has_calls(calls, any_order=True)


def test_changedmodel(mocker):
    x = mocker.patch(
        'django_amazon_translate.translator._amazonTranslate.translate_text',
        return_value={'TranslatedText': 'Nothing'}
    )

    f = TranslatedModel()
    f.title_en = "Ham"
    f.description_en = "Eggs"
    f.save()

    x.reset_mock()

    f.title_en = "Spam"
    f.save()

    # Check we haven't re-translated something we shouldn't have
    # as we didn't change the field with the word Bar in it
    assert(call('Eggs', 'en', 'de') not in x.call_args_list)

    calls = [
                call('Spam', 'en', 'de'),
                call('Spam', 'en', 'fr'),
                call('Spam', 'en', 'it'),
                call('Spam', 'en', 'es'),
            ]

    x.assert_has_calls(calls, any_order=True)


def test_override(mocker):
    x = mocker.patch(
        'django_amazon_translate.translator._amazonTranslate.translate_text',
        return_value={'TranslatedText': 'Nothing'}
    )

    f = TranslatedModel()
    f.title_en = "Ham"
    f.description_en = "Eggs"
    f.save()

    x.reset_mock()

    f.title_fr = "Spam"
    f.save()

    x.assert_not_called()



from django.db import models

from django_amazon_translate.translator import TranslationTracker


class NonTranslatedModel(models.Model):
    pass


class TranslatedModel(models.Model):
    title = models.CharField(max_length=100, null=False, blank=False)
    description = models.TextField(blank=True)

    tt = TranslationTracker()
